import pandas as pd
from rdkit import Chem
from rdkit.Chem import Draw
from matplotlib.backends.backend_pdf import PdfPages
import matplotlib.pyplot as plt
from PIL import Image, ImageDraw, ImageFont
from rdkit import RDLogger
from tqdm import tqdm

class MolBarExclusive:

    def generate_lewis_structure(self, inchi, cid):
        RDLogger.DisableLog('rdApp.*') 
        mol = Chem.MolFromInchi(inchi)
        if mol:
            structure_image = Draw.MolToImage(mol, size=(300, 150))
            draw = ImageDraw.Draw(structure_image)
            
            # Add CID text below the structure
            font = ImageFont.load_default()
            text = f"CID: {cid}"
            #text_size = draw.textsize(text, font)
            #text_position = ((structure_image.width - text_size[0]) // 2, structure_image.height)
            text_position = (100, 140)
            draw.text(text_position, text, fill="black", font=font, align="center")

            return structure_image
        else:
            return None

    def generate_combined_image(self, original_inchi, duplicate_inchi, original_cid, duplicate_cid):
        original_structure = self.generate_lewis_structure(original_inchi, original_cid)
        duplicate_structure = self.generate_lewis_structure(duplicate_inchi, duplicate_cid)

        if original_structure and duplicate_structure:
            combined_image = Image.new('RGB', (original_structure.width * 2, original_structure.height))
            combined_image.paste(original_structure, (0, 0))
            combined_image.paste(duplicate_structure, (original_structure.width, 0))
            return combined_image
        else:
            return None

    def generate_pdf(self, df, pdf_output):

        print(f"Generating PDF file with Lewis structures from InChI filtered by MolBar-exclusive duplicates to {pdf_output}")
        with PdfPages(pdf_output) as pdf:
            for index, row in tqdm(df.iterrows(), total=df.shape[0]):
                combined_image = self.generate_combined_image(
                    row['InChI_original'], row['InChI_duplicate'],
                    row['CID_original'], row['CID_duplicate']
                )
                if combined_image:
                    plt.figure()
                    plt.imshow(combined_image)
                    plt.axis('off')
                    pdf.savefig()
                    plt.close()